"""Plugin schema package for Porringer.

This package contains the schema definitions and base classes for plugins,
including environment plugins and their parameters.
"""

from porringer.core.plugin_schema.manifest import ManifestContributor

__all__ = ['ManifestContributor']
